@@ -7,7 +7,11 @@ AddCaseButton <- function(label="ADD"){
         CaseName <- enc(CaseName,encoding="UTF-8")
         AddCase(CaseName)
         CaseNamesUpdate()
-      }
+        idx <- as.character(which(.rqda$.CasesNamesWidget[] %in%  CaseName) -1) ## note the position, before manipulation of items
+        path <-gtkTreePathNewFromString(idx)
+        gtkTreeViewScrollToCell(slot(slot(.rqda$.CasesNamesWidget,"widget"),"widget"),
+                                path,use.align=TRUE,row.align = 0.05)
+    }
     }
   }
           )
@@ -113,12 +117,10 @@ MarkCaseFun <- function(){
   if (is_projOpen(env=.rqda,conName="qdacon")) {
     con <- .rqda$qdacon
     tryCatch({
-      ans <- mark(get(".openfile_gui",env=.rqda),fore.col=NULL,back.col=.rqda$back.col)
-      ## can change the color
-      if (ans$start != ans$end){ 
+      ans <- mark(get(".openfile_gui",env=.rqda),fore.col=NULL,back.col=.rqda$back.col,addButton=FALSE)
+      if (ans$start != ans$end){
         ## when selected no text, makes on sense to do anything.
         SelectedCase <- svalue(.rqda$.CasesNamesWidget)
-        ## Encoding(SelectedCase) <- "UTF-8"
         SelectedCase <- enc(SelectedCase,encoding="UTF-8")
         currentCid <-  dbGetQuery(con,sprintf("select id from cases where name=='%s'",
                                               SelectedCase))[,1]
@@ -180,9 +182,9 @@ CaseUnMark_Button<-function(label="Unmark"){
           handler=function(h,...) {
             if (is_projOpen(env=.rqda,conName="qdacon")) {
               con <- .rqda$qdacon
-              W <- tryCatch(get(h$action$widget,env=.rqda),error=function(e){})
+              W <- .rqda$.openfile_gui
               ## get the widget for file display. If it does not exist, then return NULL.
-              sel_index <- tryCatch(sindex(W),error=function(e) {})
+              sel_index <- tryCatch(sindex(W,includeAnchor=FALSE),error=function(e) {})
               ## if the not file is open, unmark doesn't work.
               if (!is.null(sel_index)) {
                 SelectedCase <- svalue(.rqda$.CasesNamesWidget)
@@ -199,35 +201,22 @@ CaseUnMark_Button<-function(label="Unmark"){
                 if (is.numeric(rowid)) for (j in rowid) {
                   dbGetQuery(con,sprintf("update caselinkage set status=0 where rowid=%i", j))
                 }
+                coding.idx <- RQDAQuery(sprintf("select selfirst,selend from coding where fid=%i and status==1",currentFid))
+                anno.idx <- RQDAQuery(sprintf("select position from annotation where fid=%i and status==1",currentFid))$position
+                allidx <- unlist(coding.idx,anno.idx)
+                if (!is.null(allidx)){
+                  startN<- sel_index$startN +  sum(allidx <= sel_index$startN)
+                  endN <- sel_index$endN +  sum(allidx <= sel_index$endN)
+                }
                 ## better to get around the loop by sqlite condition expression.
-                ClearMark(W,min=sel_index$startN,max=sel_index$endN,clear.fore.col = FALSE, clear.back.col = TRUE)
+                ClearMark(W,min=startN,max=endN,clear.fore.col = FALSE, clear.back.col = TRUE)
                 ## even for the non-current code. can improve.
               }
               }
-            }},action=list(widget=".openfile_gui")
+            }}
             )
         }
-  
-##   AddWebSearchButton <- function(label="WebSearch",CaseNamesWidget=.rqda$.CasesNamesWidget){
-##     gbutton(label,handler=function(h,...) {
-##       if (is_projOpen(env=.rqda,conName="qdacon")) {
-##         KeyWord <- svalue(CaseNamesWidget)
-##         engine <- select.list(c("Baidu","Google","Yahoo"))
-##         if (engine=="Baidu") {
-##           KeyWord <- iconv(KeyWord, from="UTF-8")
-##           browseURL(sprintf("http://www.baidu.com/s?wd=%s",paste("%",paste(charToRaw(KeyWord),sep="",collapse="%"),sep="",collapse="")))
-##         }
-##         if (engine=="Yahoo") {
-##           KeyWord <- iconv(KeyWord, from="UTF-8")
-##           browseURL(sprintf("http://search.yahoo.com/search;_ylt=A0oGkmFV.CZJNssAOK.l87UF?p=%s&ei=UTF-8&iscqry=&fr=sfp&fr2=sfp"
-##                             ,KeyWord))
-##         }
-##     if (engine=="Google")browseURL(sprintf("http://www.google.com/search?q=%s",KeyWord))
-##       }
-##     }
-##             )
-## }
-  
+
 CaseNamesWidgetMenu <- list()
 CaseNamesWidgetMenu$"Add File(s)"$handler <- function(h, ...) {
   if (is_projOpen(env = .rqda, conName = "qdacon", message = FALSE)) {
@@ -241,7 +230,7 @@ CaseNamesWidgetMenu$"Add File(s)"$handler <- function(h, ...) {
       fileoutofcase <- subset(freefile,!(id %in% fileofcase$fid))
       } else  fileoutofcase <- freefile
     if (length(fileoutofcase[['name']])==0) gmessage("All files are linked with this case.", cont=TRUE) else {
-      ##Selected <- select.list(fileoutofcase[['name']],multiple=TRUE)    
+      ##Selected <- select.list(fileoutofcase[['name']],multiple=TRUE)
     CurrentFrame <- sys.frame(sys.nframe())
     ## sys.frame(): get the frame of n
     ## nframe(): get n of current frame
@@ -264,6 +253,13 @@ CaseNamesWidgetMenu$"Case Memo"$handler <- function(h,...){
     ## see CodeCatButton.R  for definition of MemoWidget
   }
 }
+CaseNamesWidgetMenu$"Show Cases with Memo Only"$handler <- function(h,...){
+  if (is_projOpen(env=.rqda,conName="qdacon")) {
+   cnames <- RQDAQuery("select name from cases where memo is not null")$name
+   if (!is.null(cnames)) cnames <- enc(cnames,"UTF-8")
+   .rqda$.CasesNamesWidget[] <- cnames
+  }
+}
 CaseNamesWidgetMenu$"Add/modify Attributes..."$handler <- function(h,...){
   if (is_projOpen(env=.rqda,conName="qdacon")) {
     SelectedCase <- svalue(.rqda$.CasesNamesWidget)
@@ -278,8 +274,8 @@ CaseNamesWidgetMenu$"View Attributes"$handler <- function(h,...){
    viewCaseAttr()
   }
 }
-CaseNamesWidgetMenu$"Sort by created time"$handler <- function(h,...){
-CaseNamesUpdate(.rqda$.CasesNamesWidget)
+CaseNamesWidgetMenu$"Sort All by Created Time"$handler <- function(h,...){
+  CaseNamesUpdate(.rqda$.CasesNamesWidget,sortByTime = TRUE)
 }
 CaseNamesWidgetMenu$"Web Search"$Google$handler <- function(h,...){
   KeyWord <- svalue(.rqda$.CasesNamesWidget)
@@ -314,6 +310,11 @@ CaseNamesWidgetMenu$"Web Search"$Sogou$handler <- function(h,...){
 
 ## pop-up menu of .rqda$.FileofCase
 FileofCaseWidgetMenu <- list() ## not used yet.
+FileofCaseWidgetMenu$"Add To File Category ..."$handler <- function(h, ...) {
+  if (is_projOpen(env = .rqda, conName = "qdacon", message = FALSE)) {
+    AddToFileCategory(Widget=.rqda$.FileofCase,updateWidget=FALSE)
+  }
+}
 FileofCaseWidgetMenu$"Drop Selected File(s)"$handler <- function(h, ...) {
   if (is_projOpen(env = .rqda, conName = "qdacon", message = FALSE)) {
     FileOfCat <- svalue(.rqda$.FileofCase)
@@ -337,38 +338,93 @@ FileofCaseWidgetMenu$"Drop Selected File(s)"$handler <- function(h, ...) {
       }
     }
   }
-  }
+}
+FileofCaseWidgetMenu$"Delete Selected File(s)"$handler <- function(h,...){
+    if (is_projOpen(env=.rqda,conName="qdacon")) {
+        SelectedFile <- svalue(.rqda$.FileofCase)
+        Encoding(SelectedFile) <- "UTF-8"
+        for (i in SelectedFile){
+            fid <- dbGetQuery(.rqda$qdacon, sprintf("select id from source where name='%s'",i))$id
+            dbGetQuery(.rqda$qdacon, sprintf("update source set status=0 where name='%s'",i))
+            dbGetQuery(.rqda$qdacon, sprintf("update caselinkage set status=0 where fid=%i",fid))
+            dbGetQuery(.rqda$qdacon, sprintf("update treefile set status=0 where fid=%i",fid))
+            dbGetQuery(.rqda$qdacon, sprintf("update coding set status=0 where fid=%i",fid))
+        }
+        .rqda$.FileofCase[] <- setdiff(.rqda$.FileofCase[],SelectedFile)
+    }
+}
+FileofCaseWidgetMenu$"Edit Selected File"$handler <- function(h,...){
+  EditFileFun(FileNameWidget=.rqda$.FileofCase)
+}
 FileofCaseWidgetMenu$"File Memo"$handler <- function(h,...){
-        MemoWidget("File",.rqda$.FileofCase,"source")
+  MemoWidget("File",.rqda$.FileofCase,"source")
 }
-FileofCaseWidgetMenu$"Show Uncoded Files Only (sorted)"$handler <- function(h,...){
-if (is_projOpen(env=.rqda,conName="qdacon")) {
-   fid <- GetFileId(condition="case",type="uncoded")
-   FileNameWidgetUpdate(FileNamesWidget=.rqda$.FileofCase,FileId=fid)
- }
+FileofCaseWidgetMenu$"Rename selected File"$handler <- function(h,...){
+    if (is_projOpen(env=.rqda,conName="qdacon")) {
+        selectedFN <- svalue(.rqda$.FileofCase)
+        if (length(selectedFN)==0){
+            gmessage("Select a file first.",icon="error",con=TRUE)
+        }
+        else {
+            NewFileName <- ginput("Enter new file name. ",text=selectedFN, icon="info")
+            if (!is.na(NewFileName)) {
+                Encoding(NewFileName) <- "UTF-8"
+                rename(selectedFN,NewFileName,"source")
+                Fnames <- .rqda$.FileofCase[]
+                Fnames[Fnames==selectedFN] <- NewFileName
+                .rqda$.FileofCase[] <- Fnames
+            }
+        }}}
+FileofCaseWidgetMenu$"Search Files within Seleted Case"$handler <- function(h, ...) {
+    if (is_projOpen(env = .rqda, conName = "qdacon", message = FALSE)) {
+        pattern <- ginput("Please input a search pattern.",text="file like '%%'")
+        if (!is.na(pattern)){
+            Fid <- GetFileId("case")
+            tryCatch(SearchFiles(pattern,Fid=Fid,Widget=".FileofCase",is.UTF8=TRUE),error=function(e) gmessage("Error~~~."),con=TRUE)
+        }
+    }
 }
-FileofCaseWidgetMenu$"Show Coded Files Only (sorted)"$handler <- function(h,...){
-if (is_projOpen(env=.rqda,conName="qdacon")) {
-   fid <- GetFileId(condition="case",type="coded")
-   FileNameWidgetUpdate(FileNamesWidget=.rqda$.FileofCase,FileId=fid)
- }
+FileofCaseWidgetMenu$"Show ..."$"Show All by Sorted by Imported Time"$handler <- function(h,...){
+  ## UpdateFileofCaseWidget()
+  if (is_projOpen(env=.rqda,conName="qdacon")) {
+    fid <- GetFileId(condition="case",type="all")
+    FileNameWidgetUpdate(FileNamesWidget=.rqda$.FileofCase,FileId=fid)
+  }
 }
-FileofCaseWidgetMenu$"Sort by imported time"$handler <- function(h,...){
-## UpdateFileofCaseWidget()
-if (is_projOpen(env=.rqda,conName="qdacon")) {
-   fid <- GetFileId(condition="case",type="all")
-   FileNameWidgetUpdate(FileNamesWidget=.rqda$.FileofCase,FileId=fid)
- }
+FileofCaseWidgetMenu$"Show ..."$"Show Coded Files Only (sorted)"$handler <- function(h,...){
+  if (is_projOpen(env=.rqda,conName="qdacon")) {
+    fid <- GetFileId(condition="case",type="coded")
+    FileNameWidgetUpdate(FileNamesWidget=.rqda$.FileofCase,FileId=fid)
+  }
+}
+FileofCaseWidgetMenu$"Show ..."$"Show Uncoded Files Only (sorted)"$handler <- function(h,...){
+  if (is_projOpen(env=.rqda,conName="qdacon")) {
+    fid <- GetFileId(condition="case",type="uncoded")
+    FileNameWidgetUpdate(FileNamesWidget=.rqda$.FileofCase,FileId=fid)
+  }
 }
 FileofCaseWidgetMenu$"Show Selected File Property"$handler <- function(h, ...) {
   if (is_projOpen(env = .rqda, conName = "qdacon", message = FALSE)) {
-    Fid <- GetFileId("case","selected")
-    Fcat <- RQDAQuery(sprintf("select name from filecat where catid in (select catid from treefile where fid=%i and status=1) and status=1",Fid))$name
-    Case <- RQDAQuery(sprintf("select name from cases where id in (select caseid from caselinkage where fid=%i and status=1) and status=1",Fid))$name
-    if (!is.null(Fcat)) Encoding(Fcat) <- "UTF-8"
-    if (!is.null(Case)) Encoding(Case) <- "UTF-8"
-    glabel(sprintf(" File ID is %i \n File Category is %s\n Case is %s",
-                   Fid,paste(shQuote(Fcat),collapse=", "),paste(shQuote(Case),collapse=", ")),cont=TRUE)
-  }
+    ShowFileProperty(Fid=GetFileId("case","selected"))
+    }
 }
 
+##   AddWebSearchButton <- function(label="WebSearch",CaseNamesWidget=.rqda$.CasesNamesWidget){
+##     gbutton(label,handler=function(h,...) {
+##       if (is_projOpen(env=.rqda,conName="qdacon")) {
+##         KeyWord <- svalue(CaseNamesWidget)
+##         engine <- select.list(c("Baidu","Google","Yahoo"))
+##         if (engine=="Baidu") {
+##           KeyWord <- iconv(KeyWord, from="UTF-8")
+##           browseURL(sprintf("http://www.baidu.com/s?wd=%s",paste("%",paste(charToRaw(KeyWord),sep="",collapse="%"),sep="",collapse="")))
+##         }
+##         if (engine=="Yahoo") {
+##           KeyWord <- iconv(KeyWord, from="UTF-8")
+##           browseURL(sprintf("http://search.yahoo.com/search;_ylt=A0oGkmFV.CZJNssAOK.l87UF?p=%s&ei=UTF-8&iscqry=&fr=sfp&fr2=sfp"
+##                             ,KeyWord))
+##         }
+##     if (engine=="Google")browseURL(sprintf("http://www.google.com/search?q=%s",KeyWord))
+##       }
+##     }
+##             )
+## }
