@@ -8,7 +8,7 @@ CaseNamesUpdate <- function(CaseNamesWidget=.rqda$.CasesNamesWidget,sortByTime=F
   } else {
     case <- CaseName$name
     Encoding(case) <- "UTF-8"
-    if (!sortByTime) {case <- sort(case)} else {
+    if (!sortByTime) {case <- sort(case,decreasing=decreasing)} else {
     case <- case[OrderByTime(CaseName$date,decreasing=decreasing)]
     }
   }
@@ -48,19 +48,19 @@ AddFileToCaselinkage <- function(Widget=.rqda$.fnames_rqda){
   fid <- query$id
   Encoding(query$file) <- "UTF-8"
   selend <- nchar(query$file)
-  
+
   ## select a case name -> caseid
   cases <- dbGetQuery(.rqda$qdacon,"select id, name from cases where status=1")
   if (nrow(cases)!=0){
     Encoding(cases$name) <- "UTF-8"
 ##    ans <- select.list(cases$name,multiple=FALSE)
     CurrentFrame <- sys.frame(sys.nframe())
-    
+
     RunOnSelected(cases$name,multiple=FALSE,enclos=CurrentFrame,expr={
       if (Selected!=""){
         ##Selected <- iconv(Selected,to="UTF-8")
         Encoding(Selected) <- "UTF-8"
-        caseid <- cases$id[cases$name %in% Selected]       
+        caseid <- cases$id[cases$name %in% Selected]
         exist <- dbGetQuery(.rqda$qdacon,sprintf("select fid from caselinkage where status=1 and fid in (%s) and caseid=%i",paste("'",fid,"'",sep="",collapse=","),caseid))
         if (nrow(exist)!=length(fid)){
           ## write only when the selected file associated with specific case is not in the caselinkage table
@@ -77,7 +77,7 @@ AddFileToCaselinkage <- function(Widget=.rqda$.fnames_rqda){
 
 
 
-UpdateFileofCaseWidget <- function(con=.rqda$qdacon,Widget=.rqda$.FileofCase){
+UpdateFileofCaseWidget <- function(con=.rqda$qdacon,Widget=.rqda$.FileofCase,sortByTime=FALSE,...){
   Selected <- svalue(.rqda$.CasesNamesWidget)
   if (length(Selected)!=0){
     caseid <- dbGetQuery(.rqda$qdacon,sprintf("select id from cases where status=1 and name='%s'",Selected))[,1]
@@ -86,9 +86,15 @@ UpdateFileofCaseWidget <- function(con=.rqda$qdacon,Widget=.rqda$.FileofCase){
     if (nrow(Total_fid)!=0){
       items <- dbGetQuery(con,"select name,id,date from source where status==1")
       if (nrow(items)!=0) {
-        items <- items[items$id %in% Total_fid$fid,c("name","date")]
-        items <- items$name[OrderByTime(items$date)]
-        Encoding(items) <- "UTF-8"
+          if (sortByTime){
+              items <- items[items$id %in% Total_fid$fid,c("name","date")]
+              items <- items$name[OrderByTime(items$date)]
+              Encoding(items) <- "UTF-8"}
+          else{
+              items <- items[items$id %in% Total_fid$fid,c("name")]
+              Encoding(items) <- "UTF-8"
+              items <- sort(items)
+          }
       } else items <- NULL
     } else items <- NULL
   } else items <- NULL
@@ -96,21 +102,26 @@ UpdateFileofCaseWidget <- function(con=.rqda$qdacon,Widget=.rqda$.FileofCase){
 }
 
 HL_Case <- function(){
-            if (is_projOpen(env=.rqda,conName="qdacon")) {
-              con <- .rqda$qdacon
-              SelectedFile <- svalue(.rqda$.root_edit)
-              ## Encoding(SelectedFile) <- "UTF-8"
-              currentFid <-  dbGetQuery(con,sprintf("select id from source where name=='%s'",SelectedFile))[,1]
-              W <- .rqda$.openfile_gui
-              if (length(currentFid)!=0) {
-                mark_index <-
-                  dbGetQuery(con,sprintf("select selfirst,selend from caselinkage where fid=%i and status==1",currentFid))
-                if (nrow(mark_index)!=0){
-                  ClearMark(W ,0 , max(mark_index$selend),clear.fore.col = FALSE, clear.back.col = TRUE)
-                  HL(W,index=mark_index,fore.col=NULL,back.col=.rqda$back.col)
-                }
-              }
-            }
-          }
+  if (is_projOpen(env=.rqda,conName="qdacon")) {
+    SelectedFile <- svalue(.rqda$.root_edit)
+    currentFid <-  RQDAQuery(sprintf("select id from source where name=='%s'",SelectedFile))[,1]
+    if (length(currentFid)!=0) {
+      caseName <- svalue(.rqda$.CasesNamesWidget)
+      caseid <- RQDAQuery(sprintf("select id from cases where name=='%s'",caseName))[,1]
+      idx <- RQDAQuery(sprintf("select selfirst,selend from caselinkage where fid=%i and status==1 and caseid=%i",currentFid,caseid))
+      coding.idx <- RQDAQuery(sprintf("select selfirst,selend from coding where fid=%i and status==1",currentFid))
+      anno.idx <- RQDAQuery(sprintf("select position from annotation where fid=%i and status==1",currentFid))$position
+      allidx <- unlist(coding.idx,anno.idx)
+      if (nrow(idx)!=0){
+        if (!is.null(allidx)){
+          idx[,"selfirst"] <- sapply(idx[,"selfirst"],FUN=function(x) x + sum(allidx <= x))
+          idx[,"selend"] <- sapply(idx[,"selend"],FUN=function(x) x + sum(allidx <= x))
+        }
+        ClearMark(.rqda$.openfile_gui ,0 , max(idx$selend),clear.fore.col = FALSE, clear.back.col = TRUE)
+        HL(.rqda$.openfile_gui,index=idx,fore.col=NULL,back.col=.rqda$back.col)
+      }
+    }
+  }
+}
 
 
